`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2025/07/18 10:28:48
// Design Name: 
// Module Name: FX2_CDC_Loopback
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module FX2_CDC_Loopback (
    input       clk,
    input       reset_n,
    inout [7:0] fx2_fdata,  //  FX2型USB2.0芯片的SlaveFIFO的数据线
    input       fx2_flagb,  //  FX2型USB2.0芯片的端点2空标志
    input       fx2_flagc,  //  FX2型USB2.0芯片的端点6满标志
    input       fx2_ifclk,  //  FX2型USB2.0芯片的接口时钟信号

    output [1:0] fx2_faddr,  //  FX2型USB2.0芯片的SlaveFIFO的FIFO地址线
    output fx2_sloe,  //  FX2型USB2.0芯片的SlaveFIFO的输出使能信号，低电平有效
    output fx2_slwr,  //  FX2型USB2.0芯片的SlaveFIFO的写控制信号，低电平有效
    output fx2_slrd,  //  FX2型USB2.0芯片的SlaveFIFO的读控制信号，低电平有效
    output fx2_pkt_end,  //数据包结束标志信号
    output fx2_slcs,
    //output fx2_clear,  //  FX2型USB2.0芯片的清除信号

    input  SPI_CS,
    input  SPI_SCLK,
    input  SPI_MOSI,
    output SPI_MISO,
    input  Debug_Flag,

    input SW,
	  output sh_cp,
	  output st_cp,
	  output ds
);

/*
  对于CDC串口：
  Byte 0~3: 波特率（Baud Rate），DWORD，LSB First
  Byte 4: 停止位数量（Stop Bits）:
        0 = 1位，1 = 1.5位，2 = 2位
  Byte 5: 校验（Parity）:
        0 = None，1 = Odd，2 = Even，3 = Mark，4 = Space
  Byte 6: 数据位（Data Bits），通常为 8
*/
  localparam local_data = {8'h08, 8'h00, 8'h00, 8'h00, 8'h01, 8'hC2,8'h00};


    parameter Delay_Value = 16'd4096;   //状态切换延时

  reg slrd_n;
  reg slwr_n;
  reg sloe_n;

  reg [7:0] fifo_data_in;
  wire [7:0] fifo_data_out;
  reg [7:0] data_out;

  reg [1:0] faddr_n;

  parameter [1:0] loop_back_idle = 2'd0;
  parameter [1:0] loop_back_read = 2'd1;
  parameter [1:0] loop_back_wait_flagc = 2'd2;
  parameter [1:0] loop_back_write = 2'd3;

  reg [1:0] current_loop_back_state;
  reg [1:0] next_loop_back_state;

  wire fifo_full;
  wire fifo_empty;
  wire fifo_push;
  wire fifo_pop;

  //assign fx2_clear = 0;
  assign fx2_slwr  = slwr_n;
  assign fx2_slrd  = slrd_n;
  assign fx2_sloe  = sloe_n;
  assign fx2_faddr = faddr_n;

  assign fx2_slcs  = 1'b0;

  assign fx2_fdata = data_out;

  always @(*) begin
    if(((current_loop_back_state == loop_back_idle) && (delay_cnt > 8)) | (current_loop_back_state == loop_back_read))begin
      faddr_n = 2'b00;
    end else begin
      faddr_n = 2'b10;
    end
  end

  //读控制信号产生逻辑
  always @(*) begin
    if ((current_loop_back_state == loop_back_read) && (fx2_flagb == 1'b1) && (~fifo_full)) begin
      slrd_n = 1'b0;
      sloe_n = 1'b0;
    end else begin
      slrd_n = 1'b1;
      sloe_n = 1'b1;
    end
  end

  assign fifo_push = ((slrd_n == 1'b0) & (fifo_full == 1'b0));

  always @(*) begin
    if (slrd_n == 1'b0) fifo_data_in = fx2_fdata;
    else fifo_data_in = 8'd0;
  end

  //写控制信号产生逻辑
  always @(*) begin
    if ((current_loop_back_state == loop_back_write) & (fx2_flagc == 1'b1) & (fifo_empty == 1'b0))
      slwr_n <= 1'b0;
    else slwr_n <= 1'b1;
  end

  assign fifo_pop = ((slwr_n == 1'b0) & (fifo_empty == 1'b0));

  //回送模式状态机
  always @(posedge fx2_ifclk, negedge reset_n) begin
    if (reset_n == 1'b0) current_loop_back_state <= loop_back_idle;
    else current_loop_back_state <= next_loop_back_state;
  end

  //状态切换之间加点延时
  reg [31:0] delay_cnt;
  reg delay_en;
  always @(posedge fx2_ifclk, negedge reset_n) begin
    if (reset_n == 1'b0) delay_cnt <= 32'd0;
    else if (delay_en) delay_cnt <= delay_cnt + 1'd1;
    else delay_cnt <= 32'd0;
  end

  //回送模式状态机组合逻辑
  always @(*) begin
    next_loop_back_state = current_loop_back_state;
    case (current_loop_back_state)
      loop_back_idle: begin
        if ((fx2_flagb == 1'b1) && (delay_cnt >= Delay_Value - 1)) begin
          next_loop_back_state = loop_back_read;
          delay_en = 1'b0;
        end else begin
          next_loop_back_state = loop_back_idle;
          delay_en = 1'b1;
        end
      end
      loop_back_read: begin
        if (((fx2_flagb == 1'b0) || (fifo_full)) && (delay_cnt >= Delay_Value - 1)) begin
          next_loop_back_state = loop_back_wait_flagc;
          delay_en = 1'b0;
        end else begin
          next_loop_back_state = loop_back_read;
          delay_en = 1'b1;
        end
      end
      loop_back_wait_flagc: begin
        if ((fx2_flagc == 1'b1) && (delay_cnt >= Delay_Value - 1)) begin
          next_loop_back_state = loop_back_write;
          delay_en = 1'b0;
        end else begin
          next_loop_back_state = loop_back_wait_flagc;
          delay_en = 1'b1;
        end
      end
      loop_back_write: begin
        if (((fx2_flagc == 1'b0) | (fifo_empty == 1'b1)) && (delay_cnt >= Delay_Value - 1)) begin
          next_loop_back_state = loop_back_idle;
          delay_en = 1'b0;
        end else begin
          next_loop_back_state = loop_back_write;
          delay_en = 1'b1;
        end
      end
      default: next_loop_back_state = loop_back_idle;
    endcase
  end

  always @(*) begin
    if (slwr_n == 1'b1) data_out = 8'dz;
    else data_out = fifo_data_out;
  end

  wire fifo_flush;
  assign fifo_flush = 0;  //current_loop_back_state == loop_back_idle;
  //回送缓存fifo
  fifo_1024x8 fifo_inst (
      .din       (fifo_data_in),
      .write_busy(fifo_push),
      .fifo_full (fifo_full),
      .dout      (fifo_data_out),
      .read_busy (fifo_pop),
      .fifo_empty(fifo_empty),
      .fifo_clk  (fx2_ifclk),
      .reset_    (reset_n),
      .fifo_flush(fifo_flush)
  );

assign fx2_pkt_end = ((current_loop_back_state == loop_back_idle) && (delay_cnt <= 4)) ? 1'b0 : 1'b1;

  reg Send_Data_Valid;
  reg [7:0] Send_Data;
  wire Recive_Data_Valid;
  wire [7:0] Recive_Data;
  wire [15:0] Trans_Cnt;
  wire Trans_Done;
  wire Trans_Start;
  wire Trans_End;



  SPI_Slave #(
      .CPOL(1'b0),
      .CPHA(1'b0),
      .BITS_ORDER(1'b1)
  ) SPI_Slave (
      .Clk(fx2_ifclk),
      .Rst_n(reset_n),
      .Send_Data_Valid(Send_Data_Valid),
      .Send_Data(Send_Data),
      .Recive_Data_Valid(Recive_Data_Valid),
      .Recive_Data(Recive_Data),
      .Trans_Cnt(Trans_Cnt),
      .Trans_Done(Trans_Done),
      .SPI_CS(SPI_CS),
      .SPI_SCK(SPI_SCLK),
      .SPI_MOSI(SPI_MOSI),
      .SPI_MISO(SPI_MISO),
      .Trans_Start(Trans_Start),
      .Trans_End(Trans_End)
  );

  /*当出现Trans_Start时，先接收一个字节，判断bit[7]读(0)/写(1)位，并判断bit[6:0]字节长度len
  如果为写，则读取len个字节参数并存储到寄存器[7:0]Param_Reg[63:0]
  如果为读，且长度为len字节，则从Param_Reg连续读取len个字节，通过MISO发送出去
  */

  reg [7:0] Param_Reg[63:0];
  reg [6:0] len;

  localparam S_IDLE = 7'b0000001;  // 空闲
  localparam S_CMD = 7'b0000010;  // 第一个字节完成，解析
  localparam S_READ_WAIT = 7'b0000100;  // 读等待
  localparam S_READ_REG = 7'b0001000;  // 读
  localparam S_WRITE_WAIT = 7'b0010000;  // 写等待
  localparam S_WRITE_REG = 7'b0100000;  // 写
  localparam S_END = 7'b1000000;  // 结束

  reg [6:0] SM_State;

  always @(posedge fx2_ifclk or negedge reset_n) begin
    if (~reset_n) begin
      SM_State <= S_IDLE;
      len <= 7'd0;
      Send_Data_Valid <= 1'b0;
      Send_Data <= 8'h00;
      Param_Reg[0] <= local_data[7:0];
      Param_Reg[1] <= local_data[15:8];
      Param_Reg[2] <= local_data[23:16];
      Param_Reg[3] <= local_data[31:24];
      Param_Reg[4] <= local_data[39:32];
      Param_Reg[5] <= local_data[47:40];
      Param_Reg[6] <= local_data[55:48];
    end else begin
      case (SM_State)
        S_IDLE: begin
          if (Trans_Start) begin
            SM_State <= S_CMD;
          end else begin
            SM_State <= S_IDLE;
          end
        end
        S_CMD: begin
          if (Recive_Data_Valid) begin
            len <= Recive_Data[6:0];
            if (Recive_Data[7]) begin  //写寄存器
              SM_State <= S_READ_REG;
            end else begin
              SM_State <= S_WRITE_REG;
            end
          end else begin
            SM_State <= S_CMD;
          end
        end
        S_READ_WAIT: begin
          if (Recive_Data_Valid) begin
            if (Trans_Cnt >= len + 1'b1) SM_State <= S_END;
            else SM_State <= S_READ_REG;
          end else begin
            Send_Data_Valid <= 1'b0;
            SM_State <= S_READ_WAIT;
          end

        end
        S_READ_REG: begin
          if (~SPI_SCLK) begin
            Send_Data <= Param_Reg[Trans_Cnt-1];  //读取寄存器数据
            Send_Data_Valid <= 1'b1;
            SM_State <= S_READ_WAIT;
          end else begin
            Send_Data <= Param_Reg[Trans_Cnt-1];  //读取寄存器数据
            Send_Data_Valid <= 1'b0;
          end
        end
        S_WRITE_WAIT: begin
            if (Trans_Cnt >= len + 1'b1) SM_State <= S_END;
            else SM_State <= S_WRITE_REG;
        end
        S_WRITE_REG: begin
          if (Recive_Data_Valid) begin
            Param_Reg[Trans_Cnt-2] <= Recive_Data;
            SM_State <= S_WRITE_WAIT;
          end else begin
            SM_State <= S_WRITE_REG;
          end
        end
        S_END: begin
          len <= 7'd0;
          Send_Data_Valid <= 1'b0;
          Send_Data <= 8'h00;
          SM_State <= S_IDLE;
        end
        default: SM_State <= S_IDLE;
      endcase
    end
  end


	wire [7:0] sel;//数码管位选（选择当前要显示的数码管）
	wire [6:0] seg;//数码管段选（当前要显示的内容)
  wire [31:0] disp_data;

	hc595_driver hc595_driver(
		.clk(fx2_ifclk),
		.reset_n(reset_n),
		.data({1'd1,seg,sel}),
		.s_en(1'b1),
		.sh_cp(sh_cp),
		.st_cp(st_cp),
		.ds(ds)
	);
	
	hex8 hex8(
		.clk(fx2_ifclk),
		.reset_n(reset_n),
		.en(1'b1),
		.disp_data(disp_data),
		.sel(sel),
		.seg(seg)
	);

  assign disp_data = SW ? {Param_Reg[7],Param_Reg[6],Param_Reg[5],Param_Reg[4]} : {Param_Reg[3],Param_Reg[2],Param_Reg[1],Param_Reg[0]};



  // 以下部分的代码用于调试
  wire clk_96m;  //生成96M时钟用于采样usb信号

    Gowin_PLL your_instance_name(
        .clkin(clk), //input  clkin
        .init_clk(clk), //input  init_clk
        .clkout0(clk_96m) //output  clkout0
);

endmodule
